<?php
/**
 * Plugin Name: CDP & CRM CareCloud customer interface
 * Plugin URI: crmcarecloud.com
 * Description: This plugin contains components for CRM CareCloud system
 * Version: 1.4.3
 * Author: Cortex a.s.
 * Author URI: https://www.cortex.cz/
 * Text Domain: carecloud
 **/

use GuzzleHttp\Exception\BadResponseException;

if (empty(session_id())) {
  session_start();
  //var_dump($_SESSION);
  //session_destroy();
  //exit;
}

include_once __DIR__ . '/utils.php';
include_once __DIR__ . '/vendor/autoload.php';
include_once __DIR__ . '/classes/CareCloudCustomer.class.php';
include_once __DIR__ . '/classes/CareCloudAPI.class.php';
include_once __DIR__ . '/classes/CareCloudTranslations.class.php';
include_once __DIR__ . '/classes/CareCloudSettings.class.php';
include_once __DIR__ . '/classes/CareCloudSettingsTranslations.class.php';
include_once __DIR__ . '/send_email.php';

// CareCloud customer shortcodes
include_once __DIR__ . '/carecloud-shortcodes.php';

// REGISTER ACTIONS
add_action('admin_enqueue_scripts', 'loadCareCloudAdminFiles'); // Admin css and js
add_action('wp_enqueue_scripts', 'loadCareCloudFiles'); // wp public web css and js
add_action('wp_ajax_nopriv_carecloud_api', 'CRMCareCloudAPI'); // REST API endpoint
add_action('wp_ajax_carecloud_api', 'CRMCareCloudAPI'); // REST API endpoint
add_action('wp_ajax_nopriv_carecloud_user_api', 'CRMCareCloudUserAPI'); // REST API endpoint
add_action('wp_ajax_carecloud_user_api', 'CRMCareCloudUserAPI'); // REST API endpoint
add_action('wp_ajax_nopriv_carecloud_logout', 'careCloudCustomerLogout'); // Log out customer
add_action('wp_ajax_task', 'sendMailTask'); // Send task form for HZ
add_action('wp_ajax_nopriv_task', 'sendMailTask'); // Send task form for HZ

add_action('wp_ajax_nopriv_integration_layer', 'CRMCareCloudIntegrationLayer'); // Integration layer endpoint
add_action('wp_ajax_integration_layer', 'CRMCareCloudIntegrationLayer'); // Integration layer endpoint

add_action('wp_footer', 'gtm_page_view_event');

add_action('init', 'carecloud_plugin_init');

if (isset($_POST['carecloud-api-login-form-send'])) {
  add_action('init', 'api_login_controller_call');
}

if (isset($_GET['loyalty-logout'])) {
  add_action('init', 'api_logout_controller_call');
}

add_action('admin_menu', 'add_settings_to_admin_menu'); // Add item to admin menu

// REGISTER FILTERS
add_filter('body_class', 'carecloudBodyClasses');

function add_settings_to_admin_menu()
{
  $CareCloudSettings = new CareCloudSettings(CareCloudCustomer::getInstance());
  $CareCloudSettings->addSettingsToAdminMenu();
}

function carecloudBodyClasses($classes)
{

  if (CareCloudCustomer::getInstance()->careCloudAPI->isCustomerLogged())
    $classes[] = 'logged-in-carecloud';

  return $classes;

}

function loadCareCloudAdminFiles()
{
  $plugin_url = plugin_dir_url(__FILE__);

  wp_enqueue_style('wp-color-picker');
  wp_enqueue_style('crmcarecloud-admin-styles', $plugin_url . 'css/admin_index.css');
  wp_enqueue_script('crmcarecloud-admin-script', $plugin_url . 'js/admin_index.js', array('wp-color-picker'), '', true);
}

function loadCareCloudFiles()
{
  $plugin_url = plugin_dir_url(__FILE__);

  // Because of cache
  $vue_version = null;
  if (file_exists(__DIR__ . '/vue/version.txt')) {
    $vue_version = file_get_contents(__DIR__ . '/vue/version.txt');
  }

  wp_enqueue_style('crmcarecloud-styles', $plugin_url . 'css/index.css');
  wp_enqueue_script('crmcarecloud-script', $plugin_url . 'js/index.js', null, '', true);
  wp_enqueue_style('crmcarecloud-vue-styles', $plugin_url . 'vue/bundle.css', null, $vue_version);
  wp_enqueue_script('crmcarecloud-vue-script', $plugin_url . 'vue/bundle.js', null, $vue_version, true);
  wp_localize_script('crmcarecloud-script', 'carecloud_data_object',
    array('ajax_url' => admin_url('admin-ajax.php')));
}

function careCloudCustomerLogout()
{
  global $wpdb;

  session_destroy();
  $CC = CareCloudCustomer::getInstance();

  $res = $CC->careCloudAPI->logoutController();

  echo '{"result": "ok"}';

  wp_die();
}

function carecloud_plugin_init()
{
  $CC = CareCloudCustomer::getInstance();

  if (isset($_GET['redirectto']) && $CC->careCloudAPI->isCustomerLogged()) {

    header('Location: ' . get_site_url() . $_GET['redirectto']);
    echo "<script>window.location = '" . get_site_url() . $_GET['redirectto'] . "'</script>";
    wp_die();

  }

  if (CareCloudCustomer::getInstance()->careCloudAPI->isAfterLoginAccess()) {
    add_action('wp_footer', 'gtm_login_event');
  }
}

function api_login_controller_call()
{
  CareCloudCustomer::getInstance()->careCloudAPI->customerAPILoginController();
}

function api_logout_controller_call(): void
{
  CareCloudCustomer::getInstance()->careCloudAPI->logoutController();
}

function gtm_login_event(): void
{
  $customer_id = CareCloudCustomer::getInstance()->careCloudAPI->getCustomerID();
  echo "<script type=\"text/javascript\">window.addEventListener('load', function () {if(window.hasOwnProperty('dataLayer')){window.dataLayer.push({'event': 'login','login': {'method': 'credentials'},'user': {'id': '" . $customer_id . "','type': 'Customer'},_clear: true});}});</script>";
  echo "<script type=\"text/javascript\">window.addEventListener('load', function () {if(window.hasOwnProperty('cxdataLayer')){window.cxdataLayer.push({'event': 'login','login': {'method': 'credentials'},'user': {'id': '" . $customer_id . "','type': 'Customer'},_clear: true});}});</script>";
}

function gtm_page_view_event(): void
{
  $post = get_post();


  $customer_id = CareCloudCustomer::getInstance()->careCloudAPI->getCustomerID();
  $data_output = json_encode(array(
      "event" => "page_view",
      "platform" => "web",
      "environment" => "production",
      "language" => get_bloginfo('language'),
      "page" => array(
        "title" => $post->post_title,
        "url" => $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'],
        "path" => $_SERVER['REQUEST_URI']
      )
    )
  );

  echo "<script type=\"text/javascript\">window.addEventListener(\"load\", function () {if(window.hasOwnProperty(\"dataLayer\")){window.dataLayer.push($data_output)}})</script>";

}

function CRMCareCloudAPI()
{
  global $wpdb; // this is how you get access to the database

  $input_data = json_decode(file_get_contents('php://input'), true);
  $data = (isset($input_data) && isset($input_data['data'])) ? $input_data['data'] : array();

  $route = $input_data['route'];
  $method = isset($input_data['method']) ? $input_data['method'] : "GET";


  if (empty($route)) {
    echo json_encode(array('error' => 'Please fill all required parametrs.'));
    wp_die();
  }

  $CC = CareCloudCustomer::getInstance();

  try {
    $res = $CC->careCloudAPI->customerAPICall($route, $method, $data);
  } catch (BadResponseException $e) {
    $res = json_decode($e->getResponse()->getBody()->getContents());
  } catch (Exception $e) {
    $res = array('error' => 'An error has occurred.');
  }

  echo json_encode($res);

  wp_die(); // this is required to terminate immediately and return a proper response
}

function CRMCareCloudUserAPI()
{
  global $wpdb; // this is how you get access to the database

  $input_data = json_decode(file_get_contents('php://input'), true);
  $data = (isset($input_data) && isset($input_data['data'])) ? $input_data['data'] : array();

  $route = $input_data['route'] ?? '';
  $method = isset($input_data['method']) ? $input_data['method'] : "GET";

  $CC = CareCloudCustomer::getInstance();


  try {
    $res = $CC->careCloudAPI->customerAPIUserCall($route, $method, $data);
  } catch (BadResponseException $e) {
    $res = json_decode($e->getResponse()->getBody()->getContents());
  } catch (Exception $e) {
    $res = array('error' => 'An error has occurred.');
  }

  echo json_encode($res);

  wp_die(); // this is required to terminate immediately and return a proper response
}

function CRMCareCloudIntegrationLayer()
{
  global $wpdb; // this is how you get access to the database

  $input_data = json_decode(file_get_contents('php://input'), true) ?: $_POST;
  $jsonData = (isset($input_data) && isset($input_data['data'])) ? $input_data['data'] : array();

  $route = $input_data['route'] ?? '';
  $method = isset($input_data['method']) ? $input_data['method'] : "GET";
  $CC = CareCloudCustomer::getInstance();

  try {
    $gzl = new GuzzleHttp\Client([
      'base_uri' => 'https://integration-layer.crmcarecloud.com/webservice/rest-api/enterprise-interface/v1.0/',
      'timeout' => 15.0,
    ]);

    header("Content-Type: application/json");

    $data = array(
      'headers' => array(
        //'Authorization'   => 'Basic Y3VzdG9tZXJfaW50ZXJmYWNlOg==',
        'Content-Type' => 'application/json',
        'Accept-Language' => $CC->getLanguage(),
      ),
    );

    if (!empty($jsonData)) {
      array_walk_recursive($jsonData, function (&$item, $key) {
        $item = ($item === "") ? null : $item;
      });
      $data = array_merge($data, array('json' => $jsonData));
    }

    $response = $gzl->request($method, $route, $data);

    $result = json_decode($response->getBody()->getContents());

    echo json_encode($result->data);
    wp_die();

  } catch (BadResponseException $e) {
    $res = json_decode($e->getResponse()->getBody()->getContents());
  } catch (Exception $e) {
    $res = array('error' => $e->getMessage());
  }

  echo json_encode($res);

  wp_die(); // this is required to terminate immediately and return a proper response
}
