<?php

add_shortcode('carecloud-customer-menu-items', 'carecloudCustomerMenuItems');

function carecloudCustomerMenuItems($atts = [])
{
  $CC = CareCloudCustomer::getInstance();
  $login_page = $CC->getLoginPage();
  $overview_page = $CC->getCustomerOverviewPage();

  if ($CC->careCloudAPI->isCustomerLogged()) {
    return '<a href="' . get_permalink($overview_page) . '">' . get_the_title($overview_page) . '</a>';
  } else
    return '<a href="' . get_permalink($login_page) . '" class="carecloud-login-link">' . get_the_title($login_page) . '</a>';
}

add_shortcode('carecloud-customer-logout-link', 'carecloudCustomerLogoutLink');

function carecloudCustomerLogoutLink(): string
{
  return "?loyalty-logout=1";//get_site_url() .'/wp-content/plugins/carecloud-customer/logout.php';
}

add_shortcode('carecloud-customer-login-form', 'carecloudCustomerLoginForm');

function carecloudCustomerLoginForm($atts = [])
{

  $CC = CareCloudCustomer::getInstance();

  if (!$CC->pluginHasSetVars()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_setup_plugin') . '</div>';
  }

  if ($CC->careCloudAPI->isCustomerLogged() && $CC->getCustomerOverviewPage() !== null) {
    $CC->redirectToCustomerOverviewPage();
  }

  $error = "";
  if ($CC->careCloudAPI->hasErrors()) {
    $errs = $CC->careCloudAPI->getErrors();
    foreach ($errs as $err) {
      $error .= '<p class="form-error">' . $err['title'] . '</p>';
    }
  }

  $headline = $CC->getTranslation('login_to_your_customer_account');
  $headline = ($headline == "") ? "" : '<h1 class="form-title-login">' . $headline . '</h1>';


  return $headline
    . $error . '
        <form action="" method="post" class="login-form-carecloud">
		    <br/>
			
            <label class="label-login">' . $CC->getTranslation('email_address') . '</label><br/>
            <input type="text" name="carecloud-api-login-mail" class="login-field" placeholder="' . $CC->getTranslation('email_address_placeholder') . '"/>

            <br/>
    
            <label class="label-login">' . $CC->getTranslation('password') . '</label><br/>
            <input type="password" name="carecloud-api-login-password" class="login-field" placeholder="' . $CC->getTranslation('password_placeholder') . '" />
                
            <br/>
            <button type="submit" name="carecloud-api-login-form-send" class="login-button">' . $CC->getTranslation('login') . '</button>
        </form>
    ';
}

add_shortcode('carecloud-customer-voucher-validity-form', 'carecloudCustomerVoucherValidity');

function carecloudCustomerVoucherValidity($atts = [])
{

  $CC = CareCloudCustomer::getInstance();

  if (!$CC->pluginHasSetVars()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_setup_plugin') . '</div>';
  }

  /*
  if($CC->careCloudAPI->isCustomerLogged() && $CC->getCustomerOverviewPage() !== null ){
    $CC->redirectToCustomerOverviewPage();
  }*/

  $error = "";
  if ($CC->careCloudAPI->hasErrors()) {
    $errs = $CC->careCloudAPI->getErrors();
    foreach ($errs as $err) {
      $error .= '<p class="form-error">' . $err['title'] . '</p>';
    }
  }

  $headline = $CC->getTranslation('voucher_validity_headline');
  $headline = ($headline == "") ? "" : '<h1 class="form-title-login">' . $headline . '</h1>';


  return $headline
    . $error . '
        <form action="" method="post" class="voucher-validity-form-carecloud">
		    <br/>
			
            <label class="label-login">' . $CC->getTranslation('voucher_number_input_label') . '</label><br/>
            <input type="text" name="carecloud-voucher-number" class="voucher-number-field" placeholder="' . $CC->getTranslation('voucher_number_input_placeholder') . '"/>
 
            <br/>
            <button type="submit" name="carecloud-api-login-form-send" class="login-button">' . $CC->getTranslation('verify') . '</button>
        </form>
    ';
}

add_shortcode('carecloud-customer-component', 'carecloudCustomerComponent');

function carecloudCustomerComponent($atts = [])
{
  $CC = CareCloudCustomer::getInstance();

  if (!$CC->pluginHasSetVars()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_setup_plugin') . '</div>';
  }

  if (!$CC->careCloudAPI->isCustomerLogged() && $CC->getLoginPage() !== null) {
    $CC->redirectToLoginPage();
  }

  $url = (
    empty($_SESSION[CareCloudCustomer::APIURLKEY]) ||
    empty($_SESSION[CareCloudCustomer::EXTERNALAPPIDKEY]) ||
    empty($_SESSION[CareCloudCustomer::LANGUAGEKEY])
  ) ? admin_url('admin-ajax.php') : plugins_url('carecloud-customer/rest-api.php');

  $customer_id = $CC->careCloudAPI->getCustomerID();

  $data_options = array(
    "url" => $url,
    "locale" => $CC->getLanguage(),
    "customer_source" => $CC->getCustomerSource(),
    "theme" => $CC->getSkin(),
    "customer_id" => $customer_id,
    "colors" => $CC->getSkinColors(),
    "config" => json_decode($CC->getConfigJson()),
    "locales" => $CC->translations->getVueTranslations(),
    "messages" => $CC->careCloudAPI->getMessagesAsArrayForVue()
  );

  return '<div data-component="' . $atts['component'] . '" data-options="' . htmlentities(json_encode($data_options), ENT_QUOTES, 'UTF-8') . '"></div>';
}

add_shortcode('carecloud-customer-registration-form', 'carecloudCustomerRegistrationForm');

function carecloudCustomerRegistrationForm()
{
  $CC = CareCloudCustomer::getInstance();

  if (!$CC->getApiURL()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_fill_rest_api_url') . '</div>';
  }

  if (!$CC->getExternalAppID()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_fill_external_application_id') . '</div>';
  }

  $customer_id = $CC->careCloudAPI->getCustomerID();

  $data_options = array(
    "url" => admin_url('admin-ajax.php'),
    "locale" => $CC->getLanguage(),
    "customer_source" => $CC->getCustomerSource(),
    "theme" => $CC->getSkin(),
    "customer_id" => $customer_id,
    "colors" => $CC->getSkinColors(),
    "config" => json_decode($CC->getConfigJson()),
    "locales" => $CC->translations->getVueTranslations(),
    "messages" => $CC->careCloudAPI->getMessagesAsArrayForVue()
  );

  return '<div data-component="Registration" data-options="' . htmlentities(json_encode($data_options), ENT_QUOTES, 'UTF-8') . '"></div>';
}

add_shortcode('carecloud-customer-component-no-login', 'carecloudCustomerComponentNoLogin');

function carecloudCustomerComponentNoLogin($atts = [])
{
  $CC = CareCloudCustomer::getInstance();

  if (!$CC->getApiURL()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_fill_rest_api_url') . '</div>';
  }

  if ($atts['component'] === "Login" && $CC->careCloudAPI->isCustomerLogged() && $CC->getCustomerOverviewPage() !== null) {
    $CC->redirectToCustomerOverviewPage();
  }

  if (!$CC->getExternalAppID()) {
    return '<div class="message-error"><h4>' . $CC->getTranslation('something_went_wrong') . '</h4>' . $CC->getTranslation('please_fill_external_application_id') . '</div>';
  }

  $customer_id = $CC->careCloudAPI->getCustomerID();

  $data_options = array(
    "url" => admin_url('admin-ajax.php'),
    "locale" => $CC->getLanguage(),
    "customer_source" => $CC->getCustomerSource(),
    "theme" => $CC->getSkin(),
    "customer_id" => $customer_id,
    "colors" => $CC->getSkinColors(),
    "config" => json_decode($CC->getConfigJson()),
    "locales" => $CC->translations->getVueTranslations(),
    "messages" => $CC->careCloudAPI->getMessagesAsArrayForVue()
  );

  return '<div data-component="' . $atts['component'] . '" data-options="' . htmlentities(json_encode($data_options), ENT_QUOTES, 'UTF-8') . '"></div>';
}


add_shortcode('carecloud', 'carecloudShortcode');

function carecloudShortcode($atts = ['component' => null, 'login' => 'yes'])
{

  //$shortcode = (isset($atts['shortcode'])) ? $atts['shortcode'] : 'component';
  $component = $atts['component'];
  $needlogin = isset($atts['login']) && $atts['login'] === 'yes';

  if ($component === "LOGIN")
    carecloudCustomerLoginForm();
  elseif ($component === "REGISTRATION")
    carecloudCustomerRegistrationForm();
  elseif ($needlogin)
    carecloudCustomerComponent($atts);
  elseif (!$needlogin)
    carecloudCustomerComponentNoLogin($atts);
  else
    carecloudCustomerComponent($component);


}
