<?php

  include_once __DIR__.'/../vendor/autoload.php';

  use GuzzleHttp\Client;

  /**
   * Class CareCloudAPI
   */
  class CareCloudAPIAsync
  {
    const LANGUAGEKEY = 'crmcarecloud_language';
    const APIURLKEY  = 'crmcarecloud_api_url';
    const APITOKENKEY = 'crmcarecloud_api_token';
    const EXTERNALAPPIDKEY  = 'crmcarecloud_external_app_id';
    const CUSTOMERIDKEY = 'carecloud_customer_id';

    public ?Client $client;

    private string $token;

    private ?string $customer_id = null;

    private string $external_app_id;

    private string $api_url;

    private string $session_token;

    private string $language;

    private array $errors = [];

    public function __construct($api_url, $external_app_id, $session_token, $language, $customer_id = null)
    {
      $this->api_url = $api_url;
      $this->external_app_id = $external_app_id;
      $this->session_token = $session_token;
      $this->language = $language;
      $this->customer_id = $customer_id;

      $this->client = new Client([
        'base_uri' => $this->getAPIURL(),
        'timeout'  => 15.0,
      ]);
    }

    public function getApiURL () {
      return $this->api_url;
    }

    public function setError(int $errorCode, string $title, string $message)
    {
      //array_push($this->errors, array('code' => $errorCode, 'title' => $title, 'message' => $message));

      echo json_encode(array('error' => 'Something went wrong.'));
      exit;
    }

    public function customerAPICall($route, $method = "GET", $jsonData = array())
    {

      $authorization_token = base64_encode('customer_interface:'.$this->session_token);

      if($route === 'send-password-setup-email')
      {
        $route = "tokens/".$this->session_token."/actions/send-password-setup-email";
      }

      if($route === 'temporary-login')
      {
        $route = "tokens/".$this->session_token."/actions/login";
      }

      if($route === 'temporary-logout')
      {
        $route = "tokens/".$this->session_token."/actions/logout";
      }

      $data = array(
        'headers' => array(
          'Authorization'   => 'Basic '.$authorization_token,
          'Content-Type'    => 'application/json',
          'Accept-Language' => $this->language,
        ),
      );

      if(!empty($jsonData))
      {
        array_walk_recursive($jsonData, function(&$item, $key)
        {
          $item = ($item === "") ? null : $item;
        });
        $data = array_merge($data, array('json' => $jsonData));
      }

      $response = $this->client->request($method, $route, $data);

      $result = json_decode($response->getBody()->getContents());

      return $result->data;
    }

    public function customerAPIUserCall($route, $method = "GET", $data = array())
    {
      return $this->customerAPICall('customers/'.$this->customer_id."/".$route, $method, $data);
    }

    public function integrationSericeAPICall($route, $method = "GET", $data = array())
    {
      global $wpdb; // this is how you get access to the database

      $input_data = json_decode(file_get_contents('php://input'), true);
      $jsonData = (isset($input_data) && isset($input_data['data'])) ? $input_data['data'] : array();

      $route = $input_data['route'] ?? '';
      $method = isset($input_data['method']) ? $input_data['method'] : "GET";

      try
      {
        // ODLIE $res = $CC->careCloudAPI->customerAPIUserCall($route, $method, $data);
        $gzl = new GuzzleHttp\Client([
          'base_uri' => 'https://integration-layer.crmcarecloud.com/webservice/rest-api/enterprise-interface/v1.0/kro-kitchen/',
          'timeout'  => 15.0,
        ]);

        $data = array(
          'headers' => array(
            //'Authorization'   => 'Basic Y3VzdG9tZXJfaW50ZXJmYWNlOg==',
            'Content-Type'    => 'application/json',
            //'Accept-Language' => $this->CareCloud->getLanguage(),
          ),
        );

        if(!empty($jsonData))
        {
          array_walk_recursive($jsonData, function(&$item, $key)
          {
            $item = ($item === "") ? null : $item;
          });
          $data = array_merge($data, array('json' => $jsonData));
        }


        $response = $gzl->request($method, $route, $data);

        $result = json_decode($response->getBody()->getContents());

        return $result->data;

      }
      catch(\GuzzleHttp\Exception\BadResponseException $e)
      {
        $res = json_decode($e->getResponse()->getBody()->getContents());
      }
      catch(\Exception $e)
      {
        $res = array('error' => 'An error has occurred.');
      }

      return json_encode($res);

      //wp_die(); // this is required to terminate immediately and return a proper response
    }
  }