<?php

include_once 'CareCloudSettings.class.php';
include_once 'CareCloudAPI.class.php';

/**
 * Class CareCloudCustomer
 * @description Singleton class which controls
 */
class CareCloudCustomer
{
  const APIURLKEY  = 'crmcarecloud_api_url';
  const EXTERNALAPPIDKEY  = 'crmcarecloud_external_app_id';
  const CUSTOMERSOURCEKEY  = 'crmcarecloud_customer_source';
  const SKINKEY  = 'crmcarecloud_skin';
  const SKINCOLORSKEY  = 'crmcarecloud_skin_color';
  const CONFIGJSONKEY  = 'crmcarecloud_config_json';
  const LOGINPAGEKEY = 'crmcarecloud_login_page';
  const CUSTOMEROVERVIEWPAGEKEY = 'crmcarecloud_customer_overview_page';
  const LANGUAGEKEY = 'crmcarecloud_language';

  private $APIURL = null;
  private $externalAppID = null;
  private $customerSource = null;
  private $skin = null;
  private $skincolors = null;
  private $configjson = null;
  private $loginPageURL = null;
  private $customerOverviewPageURL = null;
  private $language = null;

  public $careCloudSettings = null;
  public $careCloudAPI = null;
  public $translations = null;

  private static $instance = null;

  protected function __construct()
  {
    // Check whether the language has changed
    if(function_exists('get_locale') && substr(get_locale(), 0, 2) !== $this->getLanguage()) {
      $this->getLanguage(true);
    }

    $this->translations = new CareCloudTranslations($this->getLanguage());
    $this->careCloudAPI = new CareCloudAPI($this);
  }

  public static function getInstance () : CareCloudCustomer {
    if(CareCloudCustomer::$instance === null)
      CareCloudCustomer::$instance = new CareCloudCustomer();

    return CareCloudCustomer::$instance;
  }

  public function getApiURL (bool $flush = false) : string {

    // Load from cache
    if($this->APIURL === null && isset($_SESSION[self::APIURLKEY]))
      $this->APIURL = $_SESSION[self::APIURLKEY];

    // Load from database
    if($this->APIURL === null || $flush === true)
      $this->APIURL = $_SESSION[self::APIURLKEY] = get_option(self::APIURLKEY);

    return $this->APIURL;
  }

  public function getExternalAppID (bool $flush = false) : string {

    // Load from cache
    if($this->externalAppID === null && isset($_SESSION[self::EXTERNALAPPIDKEY]))
      $this->externalAppID = $_SESSION[self::EXTERNALAPPIDKEY];

    // Load from database
    if($this->externalAppID === null || $flush === true)
      $this->externalAppID = $_SESSION[self::EXTERNALAPPIDKEY] = get_option(self::EXTERNALAPPIDKEY);

    return $this->externalAppID;
  }

  public function getLanguage(bool $flush = false) : string {

    // Load from cache
    if($this->language === null && isset($_SESSION[self::LANGUAGEKEY]))
      $this->language = $_SESSION[self::LANGUAGEKEY];

    if($this->language === null || $flush === true)
      $this->language = $_SESSION[self::LANGUAGEKEY] = substr(get_locale(), 0, 2);

    return $this->language;

  }

  public function getCustomerSource (bool $flush = false) : string {
    if($this->customerSource === null || $flush === true)
      $this->customerSource = get_option(self::CUSTOMERSOURCEKEY);

    return $this->customerSource;
  }

  public function getSkin (bool $flush = false) : string {
    if($this->skin === null || $flush === true)
      $this->skin = get_option(self::SKINKEY);

    return $this->skin;
  }

  public function getSkinColors (bool $flush = false) : array {
    if($this->skincolors === null || $flush === true)
      $this->skincolors = get_option(self::SKINCOLORSKEY);

    if(!is_array($this->skincolors))
      return array();

    // Remove '' (empty) items
    foreach($this->skincolors as $key => $value ) {
      if($value === '')
        unset($this->skincolors[$key]);
    }

    return $this->skincolors;
  }

  public function getSkinColor (string $color = '') : string {
    $colors = $this->getSkinColors();

    if($color != '' && array_key_exists($color, $colors))
      return $colors[$color];

    return '';
  }

  public function getConfigJson ( bool $flush = false) : string {
    if($this->configjson === null || $flush === true)
      $this->configjson = json_decode(get_option(self::CONFIGJSONKEY)) ?: "";

    return $this->configjson;
  }

  public function getLoginPage (bool $flush = false) {
    if($this->loginPageURL === null || $flush === true)
      $this->loginPageURL = get_option(self::LOGINPAGEKEY);

    return $this->loginPageURL;
  }

  public function getCustomerOverviewPage (bool $flush = false) {
    if($this->customerOverviewPageURL === null || $flush === true)
      $this->customerOverviewPageURL = get_option(self::CUSTOMEROVERVIEWPAGEKEY);

    return $this->customerOverviewPageURL;
  }

  public function redirectToCustomerOverviewPage ($type = 'js') {
    $url = get_permalink($this->getCustomerOverviewPage());

    if(isset($_GET['redirectto']) && $this->careCloudAPI->isCustomerLogged()) {
      $url = get_site_url() . $_GET['redirectto'];
    }

    if($type === 'header') {
      header('Location: '.$url);
    }
    else
    {
      echo "<script>window.location = '".$url."'</script>";
    }

    exit;

  }

  public function redirectToLoginPage ($type = 'js') {
    $url = get_permalink($this->getLoginPage());

    if($type === 'header') {
      header('Location: '. $url);
    }
    else
    {
      echo "<script>window.location = '".$url."?redirectto='+location.pathname;</script>";
    }

    exit;

  }


  public function pluginHasSetVars () {
    $api_url = $this->getApiURL();
    $overview_page_url = $this->getCustomerOverviewPage();
    $external_app_id = $this->getExternalAppID();
    $login_page_id = $this->getLoginPage();

    if(
      isset($overview_page_url) &&
      isset($external_app_id) &&
      isset($login_page_id) &&
      isset($api_url)
    ) {
      return true;
    }

    return false;
  }

  public function getTranslation ($key) {
    return $this->translations->getByKey($key);
  }
}