<?php

  /**
   * Class CareCloudSettings
   * @description This class controls admin page settings in wordpress
   */
  class CareCloudSettings
  {
    private $CareCloud;

    private $settingsTranslations;

    private $notice = false;

    // Customer sources options
    private $customer_sources = [];

    private $APIWrongCredentialsFlag = null;

    public function __construct(CareCloudCustomer $CareCloud)
    {
      $this->CareCloud = $CareCloud;
    }

    public function addSettingsToAdminMenu()
    {
      add_menu_page(
        __('CDP & CRM CareCloud', 'carecloud'),
        __('CDP & CRM CareCloud', 'carecloud'),
        'manage_options',
        'crmcarecloud',
        array($this, 'adminPageController'),
        'dashicons-cloud',
        76
      );

      add_submenu_page(
        'crmcarecloud',
        __('General', 'carecloud'),
        __('General', 'carecloud'),
        'manage_options',
        'crmcarecloud',
        array($this, 'adminPageController'),
        0,
      );

      add_submenu_page(
        'crmcarecloud',
        __('Text translations', 'carecloud'),
        __('Translations', 'carecloud'),
        'manage_options',
        'crmcarecloudtexts',
        array($this, 'adminTextsController'),
        1,
      );
    }

    public function adminTextsController()
    {
      $this->settingsTranslations = new CareCloudSettingsTranslations($this->CareCloud);
    }

    public function removeqsvar($url, $varname)
    {
      return preg_replace('/([?&])'.$varname.'=[^&]+(&|$)/', '$1', $url);
    }

    public function adminPageController()
    {

      if(isset($_GET['logoutuserfromcrmcarecloud']))
      {
        $this->CareCloud->careCloudAPI->logoutController(false);
        echo "<script>window.location = '".$this->removeqsvar($_SERVER['REQUEST_URI'], 'logoutuserfromcrmcarecloud')."'</script>";
        exit;
      }

      if($_SERVER["REQUEST_METHOD"] == "POST")
      {
        $this->saveFormSettings();
      }

      if(isset($_GET['logintestusertocrmcarecloud']))
      {
        $res = $this->CareCloud->careCloudAPI->loginTestUser();
        if(!$this->CareCloud->careCloudAPI->hasErrors())
        {
          $this->notice = "Successfully logged in.";
        }
      }

      if($this->CareCloud->careCloudAPI->hasErrors())
      {
        echo $this->CareCloud->careCloudAPI->printFormattedErrors();
      }

      if($this->notice)
      {
        ?>
          <div class="notice notice-success is-dismissible">
              <p><?php echo (strlen($this->notice) > 1) ? $this->notice : "Your settings have been updated!"; ?></p>
          </div>
        <?php
      }
      ?>
        <h3>Available shortcodes</h3>
        <h4>[carecloud-customer-login-form]</h4>
        <p>Includes login form into the page.</p>

        <h4>[carecloud-customer-registration-form]</h4>
        <p>Includes registration form into the page.</p>

        <h4>[carecloud-customer-logout-link]</h4>
        <p>This generates link for logout. You can use it in menus with plugin Shortcode in Menus.</p>

        <h4>[carecloud-customer-component-no-login component="Recovery"]</h4>
        <p>Shortcode for password recovery page.</p>


        <hr/>

      <?php

      $this->addSettingsForm();
    }

    public function saveFormSettings()
    {
      if(isset($_POST['crmcarecloud-api-url']) && $this->CareCloud->getApiURL() !== $_POST['crmcarecloud-api-url'])
      {
        $this->CareCloud->careCloudAPI->flushSessionToken();
        update_option($this->CareCloud::APIURLKEY, $_POST['crmcarecloud-api-url']);
        $this->CareCloud->getApiURL(true);

        // Reinitiate CareCloudAPI when settings updated
        if(!empty($this->CareCloud->getApiURL()))
          $this->CareCloud->careCloudAPI = new CareCloudAPI($this->CareCloud);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-external-app-id']) && $this->CareCloud->getExternalAppID() !== $_POST['crmcarecloud-external-app-id'])
      {
        $this->CareCloud->careCloudAPI->flushSessionToken();
        update_option($this->CareCloud::EXTERNALAPPIDKEY, $_POST['crmcarecloud-external-app-id']);
        $this->CareCloud->getExternalAppID(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-customer-source']) && $this->CareCloud->getCustomerSource() !== $_POST['crmcarecloud-customer-source'])
      {
        update_option($this->CareCloud::CUSTOMERSOURCEKEY, $_POST['crmcarecloud-customer-source']);
        $this->CareCloud->getCustomerSource(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-skin']) && $this->CareCloud->getSkin() !== $_POST['crmcarecloud-skin'])
      {
        update_option($this->CareCloud::SKINKEY, $_POST['crmcarecloud-skin']);
        $this->CareCloud->getSkin(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-skin-colors']) && $this->CareCloud->getSkinColors() !== $_POST['crmcarecloud-skin-colors'])
      {

        update_option($this->CareCloud::SKINCOLORSKEY, $_POST['crmcarecloud-skin-colors']);

        $this->CareCloud->getSkinColors(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-config-json']) && $this->CareCloud->getConfigJson() !== $_POST['crmcarecloud-config-json'])
      {
        update_option($this->CareCloud::CONFIGJSONKEY, json_encode(stripslashes($_POST['crmcarecloud-config-json'])));
        $this->CareCloud->getConfigJson(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-login-page']) && $this->CareCloud->getLoginPage() !== $_POST['crmcarecloud-login-page'])
      {
        update_option($this->CareCloud::LOGINPAGEKEY, $_POST['crmcarecloud-login-page']);
        $this->CareCloud->getLoginPage(true);

        $this->notice = true;
      }

      if(isset($_POST['crmcarecloud-customer-overview-page']) && $this->CareCloud->getCustomerOverviewPage() !== $_POST['crmcarecloud-customer-overview-page'])
      {
        update_option($this->CareCloud::CUSTOMEROVERVIEWPAGEKEY, $_POST['crmcarecloud-customer-overview-page']);
        $this->CareCloud->getCustomerOverviewPage(true);

        $this->notice = true;
      }
    }

    public function addSettingsForm()
    {
      $APIHasSetVarsFlag = $this->CareCloud->careCloudAPI->APIHasSetBasicVars(true);
      $this->APIWrongCredentialsFlag = false;

      try
      {
        if($APIHasSetVarsFlag)
          $this->CareCloud->careCloudAPI->getSessionToken();
      }
      catch(\GuzzleHttp\Exception\ConnectException $e)
      {
        $this->APIWrongCredentialsFlag = true;
      }
      catch(\Exception $e)
      {
        $this->APIWrongCredentialsFlag = true;
        //trigger_error($e->getCode()." ".$e->getMessage(), E_USER_ERROR);
      }

      if($APIHasSetVarsFlag && $this->APIWrongCredentialsFlag === false)
      {
        $customer_sources = $this->CareCloud->careCloudAPI->customerAPICall('customer-sources');
        if($customer_sources !== null && property_exists($customer_sources, 'customer_sources'))
        {
          $this->customer_sources = $customer_sources->customer_sources;
        }
      }

      $this->getSettingsForm();

      ?>
        <form method="post" action="">
          <?php
            settings_fields('carecloud_settings');
            do_settings_sections('carecloud_settings_general');

            submit_button();
          ?>
        </form>

        <hr/>
        <h2>Login test customer</h2>
        <p>You can login test customer for development purpose.</p>
      <?php if($this->CareCloud->careCloudAPI->isCustomerLogged()):
      $first_name = $_SESSION['carecloud_test_customer']['first_name'] ?: "";
      $last_name = $_SESSION['carecloud_test_customer']['last_name'] ?: "";

      echo "<p>Test user <strong>$first_name $last_name</strong> is logged in.</p>";
      ?>

        <a href="<?php echo $_SERVER['REQUEST_URI']; ?>&amp;logoutuserfromcrmcarecloud=1" class="button button-primary">Logout</a>
    <?php else: ?>
        <a href="<?php echo $_SERVER['REQUEST_URI']; ?>&amp;logintestusertocrmcarecloud=1" class="button button-primary">Login</a>
    <?php endif;
    }

    public function getSkinInput($name)
    {
      add_settings_field(
        'cc_'.$name.'_color', // ID
        $name.' color for light skin', // Title
        function() use ($name)
        {
          echo '<input type="text" id="crmcarecloud-'.$name.'-color" name="crmcarecloud-skin[secondary]" class="color-picker" data-default-color="#424242" value=""/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );
    }

    public function getSettingsForm()
    {
      register_setting(
        'carecloud_settings', // Option group
        'my_option_name' // Option name
      //array( $this, 'sanitize' ) // Sanitize
      );

      add_settings_section(
        'cc_setting_general', // ID
        __('General settings', 'crmcarecloud'), // Title
        function()
        {
          echo 'You can generate this API access data in your CRM CareCloud settings or contact CRM CareCloud support.';

          if($this->APIWrongCredentialsFlag === true)
            echo '<br/><br/><div class="notice notice-error dib"><p><strong>There has been an Error.</strong> We couldn\'t connect to CRM CareCloud API. Check your credentials.</p></div>';
        }, // Callback
        'carecloud_settings_general' // Page
      );

      add_settings_field(
        'cc_api_url', // ID
        'URL to CRM CareCloud API', // Title
        function()
        {
          echo '<input type="text" id="crmcarecloud-api-url" name="crmcarecloud-api-url" class="regular-text" value="'.$this->CareCloud->getApiURL().'"/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_external_app_id', // ID
        'External application ID', // Title
        function()
        {
          echo '<input type="text" id="crmcarecloud-external-app-id" name="crmcarecloud-external-app-id" class="regular-text" value="'.$this->CareCloud->getExternalAppID().'"/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_customer_source', // ID
        'Customer sources', // Title
        function()
        {
          echo '<select id="crmcarecloud-customer-source" name="crmcarecloud-customer-source" class="regular-text" '.(($this->CareCloud->careCloudAPI->APIHasSetBasicVars()) ?: 'disabled').'>';
          foreach($this->customer_sources as $item)
          {
            $selected = '';

            if($this->CareCloud->getCustomerSource() === $item->customer_source_id)
              $selected = 'selected';

            if($item->state === 1 /* source is active*/)
              echo '<option value="'.$item->customer_source_id.'" '.$selected.'>'.$item->name.'</option>';
          }
          echo '</select>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general', // Section
      );

      add_settings_field(
        'cc_skin', // ID
        'Choose skin', // Title
        function()
        {
          echo '<select id="crmcarecloud-skin" name="crmcarecloud-skin" class="regular-text" value="'.$this->CareCloud->getSkin().'">
                <option value="light" '.($this->CareCloud->getSkin() === 'light' ? 'selected' : '').'>Light</option>
                <option value="dark" '.($this->CareCloud->getSkin() === 'dark' ? 'selected' : '').'>Dark</option>
            </select>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_primary_color', // ID
        'Primary color for light skin', // Title
        function()
        {

          echo '<input type="text" id="crmcarecloud-primary-color" name="crmcarecloud-skin-colors[primary]" class="color-picker" data-default-color="#2196f3" value="'.$this->CareCloud->getSkinColor('primary').'"/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_secondary_color', // ID
        'Secondary color for light skin', // Title
        function()
        {
          echo '<input type="text" id="crmcarecloud-secondary-color" name="crmcarecloud-skin-colors[secondary]" class="color-picker" data-default-color="#424242" value="'.$this->CareCloud->getSkinColor('secondary').'"/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_accent_color', // ID
        'Accent color for light skin', // Title
        function()
        {
          echo '<input type="text" id="crmcarecloud-accent-color" name="crmcarecloud-skin-colors[accent]" class="color-picker" data-default-color="#82B1FF" value="'.$this->CareCloud->getSkinColor('accent').'"/>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_field(
        'cc_config_json', // ID
        'Config for Vue components in json', // Title
        function()
        {
          echo '<textarea id="crmcarecloud-config-json" class="regular-text" name="crmcarecloud-config-json">'.$this->CareCloud->getConfigJson().'</textarea>';
        }, // Callback
        'carecloud_settings_general', // Page
        'cc_setting_general' // Section
      );

      add_settings_section(
        'cc_setting_pages', // ID
        'Set basic pages for the clients portal', // Title
        function()
        {
          echo 'These pages are necessary for redirect after login and logout.';
        }, // Callback
        'carecloud_settings_general' // Page
      );

      add_settings_field(
        'title',
        'Login page',
        function()
        {
          wp_dropdown_pages(array(
            'id'               => 'crmcarecloud-login-page',
            'name'             => 'crmcarecloud-login-page',
            'sort_column'      => 'menu_order',
            'sort_order'       => 'ASC',
            'selected'         => $this->CareCloud->getLoginPage(),
            'show_option_none' => __('Choose page', 'carecloud'),
          ));
        },
        'carecloud_settings_general',
        'cc_setting_pages'
      );

      add_settings_field(
        'title2',
        'Customer overview page',
        function()
        {
          wp_dropdown_pages(array(
            'id'               => 'crmcarecloud-customer-overview-page',
            'name'             => 'crmcarecloud-customer-overview-page',
            'sort_column'      => 'menu_order',
            'sort_order'       => 'ASC',
            'selected'         => $this->CareCloud->getCustomerOverviewPage(),
            'show_option_none' => __('Choose page', 'carecloud'),
          ));
        },
        'carecloud_settings_general',
        'cc_setting_pages'
      );
    }
  }