<?php

/**
 * Class CareCloudSettings
 * @description This class controls admin page settings in wordpress
 */
class CareCloudSettingsTranslations
{
  private $CareCloud;

  private $notice = false;

  public function __construct(CareCloudCustomer $CareCloud)
  {
    $this->CareCloud = $CareCloud;

    $this->CareCloud->translations->updateTranslatedFile();
    $this->CareCloud->translations->updateTranslatedFile(
            $this->CareCloud->translations::ORIGINAL_TRANSLATION_VUE_FILE_NAME,
            $this->CareCloud->translations::TRANSLATION_VUE_FILE_NAME
    );

    $this->adminPageController();
  }

  public function adminPageController()
  {
    if($_SERVER["REQUEST_METHOD"] == "POST") {
      if (isset($_POST['crmcarecloud-locales-json'])) {
        file_put_contents($this->CareCloud->translations->getTranslationFilePath() . $this->CareCloud->translations::TRANSLATION_FILE_NAME, stripslashes($_POST['crmcarecloud-locales-json']));
      }


      // Save JSON Vue language file
      if (isset($_POST['crmcarecloud-vue-locales-json'])) {
        file_put_contents($this->CareCloud->translations->getTranslationFilePath() . $this->CareCloud->translations::TRANSLATION_VUE_FILE_NAME, stripslashes($_POST['crmcarecloud-vue-locales-json']));
      }
    }


    if($this->notice)
    {
      ?>
        <div class="notice notice-success is-dismissible">
            <p><?php $this->CareCloud->getTranslation('settings_updated'); ?></p>
        </div>
      <?php
    }
    ?>

    <?php

    $this->addSettingsForm();
  }

  public function addSettingsForm()
  {
    add_settings_section(
      'cc_setting_texts',
      __('Language settings', 'carecloud'),
      function()
      {
        echo 'Language translations for CRM CareCloud plugin.';
      },
      'carecloud_settings_content_texts'
    );

    add_settings_field(
      'cc_locales_json', // ID
      'Translations for CRM CareCloud plugin in json', // Title
      function()
      {
        echo '<textarea rows="20" id="crmcarecloud-locales-json" class="large-text" name="crmcarecloud-locales-json">' . $this->CareCloud->translations->getVueRawTranslations($this->CareCloud->translations::TRANSLATION_FILE_NAME) . '</textarea>';
      }, // Callback
      'carecloud_settings_content_texts', // Page
      'cc_setting_texts' // Section
    );

    register_setting(
      'carecloud_content_vue_texts', // Option group
      'my_option_name' // Option name
    //array( $this, 'sanitize' ) // Sanitize
    );

    add_settings_section(
      'cc_setting_vue_texts',
      __('Language Vue settings', 'carecloud'),
      function()
      {
        echo 'Language translations for Vue.';
      },
      'carecloud_settings_content_vue_texts'
    );

    add_settings_field(
      'cc_vue_locales_json', // ID
      'Vue translations for components in json', // Title
      function()
      {
        echo '<textarea rows="20" id="crmcarecloud-vue-locales-json" class="large-text" name="crmcarecloud-vue-locales-json">' . $this->CareCloud->translations->getVueRawTranslations($this->CareCloud->translations::TRANSLATION_VUE_FILE_NAME) . '</textarea>';
      }, // Callback
      'carecloud_settings_content_vue_texts', // Page
      'cc_setting_vue_texts' // Section
    );


    ?>
      <form method="post" action="">
        <?php
        settings_fields('carecloud_content_texts');
        do_settings_sections('carecloud_settings_content_texts');

        submit_button();
        ?>
      </form>

      <form method="post" action="">
        <?php
          settings_fields('carecloud_content_vue_texts');
          do_settings_sections('carecloud_settings_content_vue_texts');

          submit_button();
        ?>
      </form>
    <?php
  }
}