<?php

class CareCloudTranslations
{
  const ORIGINAL_TRANSLATION_FILE_NAME = "/../translations.json";
  const ORIGINAL_TRANSLATION_VUE_FILE_NAME = "/../vue/locales.json";
  const TRANSLATION_FILE_NAME = "translations.json";
  const TRANSLATION_VUE_FILE_NAME = "vue_locales.json";
  const OPTION_TRANSLATION_KEY = 'crmcarecloud_translations';

  private $translation = null;
  private $vuetranslations = null;

  private $activeLangugage = null;

  private $flag_hasChanged = false;

  public function __construct($activeLanguage)
  {

    $this->activeLangugage = $activeLanguage;
    $uploaddir = $this->getTranslationFilePath();
    $originaldir = __DIR__;

    // Check if file exists if not create it
    if( !file_exists($uploaddir . self::TRANSLATION_FILE_NAME ) ) {

      $original_translation_file = $this->readTranslationFile($originaldir . self::ORIGINAL_TRANSLATION_FILE_NAME);
      $this->createFileAndPath($uploaddir . self::TRANSLATION_FILE_NAME, json_encode($original_translation_file));

    }

    // Same but for locales for vue components
    if( !file_exists($uploaddir . self::TRANSLATION_VUE_FILE_NAME ) ) {
      //file_get_contents($filepath)
      $original_vue_translation_file = file_get_contents($originaldir . self::ORIGINAL_TRANSLATION_VUE_FILE_NAME);
      $this->createFileAndPath($uploaddir . self::TRANSLATION_VUE_FILE_NAME, $original_vue_translation_file);

    }

    $this->loadTranslations();
  }

  public function updateTranslatedFile ($original_file_name = self::ORIGINAL_TRANSLATION_FILE_NAME, $file_name = self::TRANSLATION_FILE_NAME) {
    $originaldir = __DIR__;
    $uploaddir = $this->getTranslationFilePath();

    $original_translation_file = $this->readTranslationFile($originaldir . $original_file_name);
    $translation_file = $this->readTranslationFile($uploaddir . $file_name);

    $new_translations = array_deep_merge($translation_file, $original_translation_file);

    file_put_contents($uploaddir . $file_name, json_encode($new_translations, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT));
  }

  public function getTranslationFilePath () {
    $dir = _wp_upload_dir()["basedir"] ?: '';

    $dir .= '/crmcarecloud/';

    return $dir;
  }

  public function getTranslations () {
    return $this->translation;
  }

  public function getVueTranslations () {
    return $this->vuetranslations;
  }

  // TODO: Rename, it is not used just for Vue
  public function getVueRawTranslations ( $filename ) {
    return file_get_contents( $this->getTranslationFilePath() . $filename);
  }

  public function readTranslationFile ( $filepath = null ) : array {

    if(file_exists($filepath))
      return json_decode(file_get_contents($filepath), true);
    else
      return array();
  }

  public function loadTranslations ($filename = self::TRANSLATION_FILE_NAME ) {

    $filepath = $this->getTranslationFilePath() . $filename;

    $this->translation = json_decode(file_get_contents( $filepath), true);
    $this->vuetranslations = json_decode(file_get_contents( $this->getTranslationFilePath() . self::TRANSLATION_VUE_FILE_NAME), true);

    return $this->translation;
  }

  public function createFileAndPath($filepath, $data = '', $chmod = 0777) {
    $pathToFile = $filepath;
    $fileName = basename($pathToFile);
    $folders = explode('/', str_replace('/' . $fileName, '', $pathToFile));

    $currentFolder = '';
    foreach ($folders as $folder) {
      $currentFolder .= $folder . DIRECTORY_SEPARATOR;
      if (!file_exists($currentFolder)) {
        mkdir($currentFolder, 0755);
      }
    }

    file_put_contents($pathToFile, $data);
    chmod($pathToFile, $chmod);
  }

  public function saveTranslationFile ( $filepath = null, $content = null ) {
    if($filepath === null)
      $filepath = $this->getTranslationFilePath() . self::TRANSLATION_FILE_NAME;

    if($content === null)
      $content = json_encode($this->translation);

    return file_put_contents($filepath, $content);
  }

  public function getByKey (string $key) {
    if(array_key_exists($key, $this->translation[$this->activeLangugage])) {
        return $this->translation[$this->activeLangugage][$key];
    } else
      return null;
  }

  public function setByKey (string $key, string $value) {
    $this->translation[$key] = $value;
    $this->flag_hasChanged = true;
  }

  public function __destruct()
  {
    if($this->flag_hasChanged)
      $this->saveTranslationFile();
  }
}